]> ruderich.org/simon Gitweb - config/dotfiles.git/blobdiff - xmonad.hs
xmonad.hs: Readd smartBorders to Full layout.
[config/dotfiles.git] / xmonad.hs
index c59bc53dce2e26d0220e6e39aecebd637541431f..cf79ee4caf726f840959906e12db615c7c18a84e 100644 (file)
--- a/xmonad.hs
+++ b/xmonad.hs
@@ -3,7 +3,8 @@
 -- Should work fine with XMonad 0.7 and above.
 
 
-import XMonad
+-- "hiding" necessary for LayoutCombinators.
+import XMonad hiding ( (|||) )
 -- Necessary for reflectVert.
 import XMonad.Layout.Reflect
 -- Necessary for avoidMaster.
@@ -20,13 +21,17 @@ import XMonad.Util.EZConfig
 import XMonad.Hooks.SetWMName
 -- Necessary for toggleWS.
 import XMonad.Actions.CycleWS
+-- Necessary for named.
+import XMonad.Layout.Named
+-- Necessary for JumpToLayout.
+import XMonad.Layout.LayoutCombinators
 
 
 -- Prevent new windows from spawning in the master pane. Taken from
 -- http://haskell.org/haskellwiki/Xmonad/Frequently_asked_questions on
 -- 2009-06-30. Thanks. Modified to not steal focus from the master pane when a
 -- new window is created, thanks to vav in #xmonad on Freenode (2010-04-15
--- 21:59).
+-- 21:59 CEST).
 avoidMaster :: W.StackSet i l a s sd -> W.StackSet i l a s sd
 avoidMaster = W.modify' $ \c -> case c of
     W.Stack t [] (r:rs) -> W.Stack r [] (t:rs)
@@ -34,19 +39,24 @@ avoidMaster = W.modify' $ \c -> case c of
 
 -- Create my custom layout.
 --
--- Only use horizontal (Mirror tiled) and fullscreen layouts.
+-- Only use horizontal (Mirror tiled) and fullscreen layouts, but allow
+-- switching to other layouts with bindings.
 --
 -- The master pane is at the top of the screen. To make sure new windows don't
 -- spawn in the master pane avoidMaster (see below) is used. Borders are only
 -- drawn when the screen has more then one window (smartBorders).
 --
--- Thanks to jrick in #xmonad on Freenode (2009-06-29 22:19) for telling me
--- how to remove the vertical tiled layout.
+-- Thanks to jrick in #xmonad on Freenode (2009-06-29 22:19 CEST) for telling
+-- me how to remove the vertical tiled layout.
 --
--- toggleLayouts is used to allow switching to vertical tiled layout but not
--- using it with meta-space (see below in key bindings). Thanks to ml| in
--- #xmonad on Freenode (2010-06-14 09:49) how to use toggleLayouts.
-myLayoutHook = toggleLayouts tiled $ smartBorders $ Mirror tiled ||| Full
+-- JumpToLayout (from LayoutCombinators) is used to jump to specific layouts,
+-- thanks to aavogt in #xmonad on Freenode (2011-06-12 22:13 CEST).
+--
+-- named is used to name layouts which allows switching to a specific layout
+-- (see below), thanks to vav in #xmonad on Freenode (2011-06-12 22:28 CEST).
+myLayoutHook = named "Default" (smartBorders $ Mirror tiled)
+           ||| named "Full" (smartBorders Full)
+           ||| named "Vertical" tiled
     where
     -- Default tiling algorithm partitions the screen into two panes.
     tiled   = Tall nmaster delta ratio
@@ -59,10 +69,10 @@ myLayoutHook = toggleLayouts tiled $ smartBorders $ Mirror tiled ||| Full
 
 -- Don't spawn new windows in the master pane (which is at the top of the
 -- screen). Thanks to dschoepe, aavogt and especially vav in #xmonad on
--- Freenode (2009-06-30 02:10f).
+-- Freenode (2009-06-30 02:10f CEST).
 --
 -- Also some applications are spawned on specific workspaces. Thanks to
--- dschoepe and ivanm in #xmonad on Freenode (2009-07-12 14:50).
+-- dschoepe and ivanm in #xmonad on Freenode (2009-07-12 14:50 CEST).
 myManageHook = composeOne
     -- Browser on "2".
     [ className =? "Iceweasel"          -?> doF (W.shift "2")
@@ -78,16 +88,41 @@ myManageHook = composeOne
     , return True -?> doF avoidMaster
     -- Prevent windows which get moved to other workspaces from removing the
     -- focus of the currently selected window. Thanks to vav in #xmonad on
-    -- Freenode (2010-04-15 21:04).
+    -- Freenode (2010-04-15 21:04 CEST).
     , return True -?> doF W.focusDown
     ]
 
+-- Switch to next layout, but skip all layouts not in layouts argument. This
+-- allows switching to some layouts with mappings but excluding them from
+-- meta-space (which gets mapped to this function). Thanks to aavogt in
+-- #xmonad on Freenode for this function (2011-06-13 12:45 CEST) and
+-- rootzlevel in #xmonad on Freenode for fixes (2011-06-13 15:20 CEST),
+-- modified to take list of layouts to switch to, not layouts to exclude.
+nextLayoutIncluding :: [String] -> X ()
+nextLayoutIncluding layouts = do
+    cws <- gets (W.workspace . W.current . windowset)
+    sendMessageWithNoRefresh NextLayout cws
+    nextLayoutIncluding' layouts
+
+nextLayoutIncluding' :: [String] -> X ()
+nextLayoutIncluding' layouts = do
+    cws <- gets (W.workspace . W.current . windowset)
+    if not $ (description $ W.layout cws) `elem` layouts
+        -- Skip over excluded layouts.
+        then do
+            sendMessageWithNoRefresh NextLayout cws
+            nextLayoutIncluding' layouts
+        -- Found allowed layout, show it.
+        else refresh
+
 myKeys = [
-        -- Switch to additional layout (vertical tiled) defined by
-        -- toggleLayouts above.
-        ((mod4Mask, xK_v), sendMessage ToggleLayout)
+        -- Switch to next layout, but only use the listed layouts.
+        ((mod4Mask, xK_space), nextLayoutIncluding ["Default", "Full"])
+        -- Switch to vertical tiled layout.
+      , ((mod4Mask, xK_v), sendMessage $ JumpToLayout "Vertical")
+
         -- Switch to last active workspace, thanks to moljac024 in #xmonad on
-        -- Freenode (2010-12-18 14:45).
+        -- Freenode (2010-12-18 14:45 CET).
       , ((mod4Mask, xK_f), toggleWS)
     ]
     ++
@@ -97,10 +132,10 @@ myKeys = [
     -- code changes it to just switch to the screen where the workspace is
     -- already displayed.
     --
-    -- Thanks to the Xmonad FAQ, read on 2010-06-16 13:42
+    -- Thanks to the Xmonad FAQ, read on 2010-06-16 13:42 CEST
     -- (http://www.haskell.org/haskellwiki/Xmonad/Frequently_asked_questions).
-    -- Thanks to MrElendig in #xmonad on Freenode (2010-06-17 17:16) to use
-    -- the default XMonad workspaces.
+    -- Thanks to MrElendig in #xmonad on Freenode (2010-06-17 17:16 CEST) to
+    -- use the default XMonad workspaces.
     [((m .|. mod4Mask, k), windows $ f i)
         | (i, k) <- zip (XMonad.workspaces defaultConfig) [xK_1 .. xK_9]
         , (f, m) <- [(W.view, 0), (W.shift, shiftMask)]]